using System;
using System.Collections.Generic;
using System.Text;
using Filters.ApplicationObjectInterface;
using System.Drawing;
using System.Drawing.Imaging;

namespace CustomFilterExample1
{
    public class CustomFilterExample1 : IFPFilter, IFPFilterMethods
    {

        public CustomFilterExample1()
        {
            _name = "CustomFilterExample";
            _filterCategory = "Custom Filters";
            _description = "Filters an image ...";
            _filterDefinition = "Parameter:0;Action:GrayScale;Parameter:1;Action:Sepia;Parameter:2;Action:Threshold 0.5";
            _fileName = "CustomFilterExample1.dll";
            _supportedPixelFormats = new List<string>();
            _supportedPixelFormats.Add(AppFilterConstants.FILTER_24BPP);
            _convertToThisPixelFormat = "24BPP";
            _filterType = AppFilterConstants.FILTER_TYPE_DLL;
            _pathToDll = _fileName;
            _filterGuid = "01b4ae5a-053d-4b7a-97bd-3e73865e1641";
            _defaultParameters = "0";//This should specify the actual default parameter which will be used by the filter if no parameter is supplied
            _version = "1.0";
        }

        string _defaultParameters;

        /// <summary>
        /// Filters an image (passed as a Bitmap)
        /// </summary>
        /// <param name="image">The image to be filtered</param>
        /// <param name="param">An object containing parameters for the actual filters. 
        /// If this is null then a default value will be considered.</param>
        /// <returns>Returns the filtered image.</returns>
        public Bitmap RunFilter(Bitmap image, object param)
        {
            
            Bitmap filteredBitmap = null;

            try
            {
                filteredBitmap = FilterBitmap(image, param);
            }
            catch (Exception ex)
            {
                _errorMessage = ex.StackTrace + " " + ex.Message;
            }

            return filteredBitmap;
        }

        public static Bitmap FilterBitmap(Bitmap bmp, object filterSelection)
        {
            //bmp.Save(@"c:\CustomFilterExample1_received_pic" + DateTime.Now.Hour + "_" + DateTime.Now.Minute + "_" + DateTime.Now.Second + ".bmp", ImageFormat.Bmp);
            Color tmpColor;
            int red, green, blue;
            int r, g, b = 0;
            string filterTarget = "0";
            if (filterSelection == null)
                filterTarget = "0";
            else
                filterTarget = filterSelection.ToString();

            //Ready the whole bitmap for reading and writing   
            BitmapData bmpData = bmp.LockBits(new Rectangle(0, 0, bmp.Width, bmp.Height), ImageLockMode.ReadWrite, bmp.PixelFormat);
            unsafe
            {
                //Sets the pointer to the first pixel      
                byte* ptr = (byte*)bmpData.Scan0;
                int bytesPerPixel = BytesPerPixel(bmp.PixelFormat);
                /*Line offset for pointer.*/
                int remain = bmpData.Stride - bmpData.Width * bytesPerPixel;
                //Loops over each pixel
                for (int y = 0; y < bmpData.Height; y++)
                {
                    for (int x = 0; x < bmpData.Width; x++)
                    {
                        b = (int)ptr[0];
                        g = (int)ptr[1];
                        r = (int)ptr[2];

                        if (filterTarget.Equals("0"))
                        {
                            //Grayscale
                            int luma = (int)(r * 0.3) + (int)(g * 0.59) + (int)(b * 0.11);
                            r = luma;
                            g = luma;
                            b = luma;
                        }
                        else if (filterTarget.Equals("1"))
                        {
                            //Sepia
                            red = (int)((r * .393) + (g * .769) + (b * .189));

                            green = (int)((r * .349) + (g * .686) + (b * .168));

                            blue = (int)((r * .272) + (g * .534) + (b * .131));

                            r = red;
                            g = green;
                            b = blue;

                            if (r > 255)
                                r = 255;
                            if (g > 255)
                                g = 255;
                            if (b > 255)
                                b = 255;
                        }
                        else if (filterTarget.Equals("2"))
                        {
                            //Threshold 0.5
                            tmpColor = Color.FromArgb(r, g, b);
                            if (tmpColor.GetBrightness() > 0.5)
                            {
                                r = 255;
                                g = 255;
                                b = 255;
                            }
                            else
                            {
                                r = 0;
                                g = 0;
                                b = 0;
                            }
                        }

                        ptr[0] = (byte)b;
                        ptr[1] = (byte)g;
                        ptr[2] = (byte)r;
                        ptr += bytesPerPixel;
                    }
                    ptr += remain;
                }
            }
            bmp.UnlockBits(bmpData);
            return bmp;
        }


        public static int BytesPerPixel(PixelFormat format)
        {
            switch (format)
            {
                case PixelFormat.Format24bppRgb:
                    return 3;
                case PixelFormat.Format8bppIndexed:
                    return 1;
                case PixelFormat.Format16bppRgb555:
                case PixelFormat.Format16bppRgb565:
                case PixelFormat.Format16bppArgb1555:
                case PixelFormat.Format16bppGrayScale:
                    return 2;
                case PixelFormat.Format32bppRgb:
                case PixelFormat.Format32bppArgb:
                case PixelFormat.Format32bppPArgb:
                    return 4;

                case PixelFormat.Format48bppRgb:
                    return 6;

                case PixelFormat.Format64bppArgb:
                case PixelFormat.Format64bppPArgb:
                    return 8;

                default:
                    throw new Exception();
            }
        }

        #region Properties

        public string ErrorMessage
        {
            get
            {
                return _errorMessage;
            }
            set
            {
                _errorMessage = value;
            }
        }

        public string _errorMessage;

        public string Name
        {
            get
            {
                return _name;
            }
            set
            {
                _name = value;
            }
        }
        string _name;

        public string FilterCategory
        {
            get
            {
                return _filterCategory;
            }
            set
            {
                _filterCategory = value;
            }
        }
        string _filterCategory;

        public string Description
        {
            get
            {
                return _description;
            }
            set
            {
                _description = value;
            }
        }
        string _description;

        public object FilterDefinition
        {
            get
            {
                return _filterDefinition;
            }
            set
            {
                _filterDefinition = value;
            }
        }
        object _filterDefinition;

        public string FileName
        {
            get
            {
                return _fileName;
            }
            set
            {
                _fileName = value;
            }
        }
        string _fileName;

        public List<string> SupportedPixelFormats
        {
            get
            {
                return _supportedPixelFormats;
            }
            set
            {
                _supportedPixelFormats = value;
            }
        }
        List<string> _supportedPixelFormats;

        public string SupportedPixelFormatsAsString
        {
            get
            {
                StringBuilder result = new StringBuilder();
                if (_supportedPixelFormats != null)
                {
                    bool first = true;
                    foreach (string str in _supportedPixelFormats)
                    {
                        if (first)
                        {
                            first = false;
                            result.Append(str);
                        }
                        else
                        {
                            result.Append("," + str);
                        }
                    }
                }
                return result.ToString();
            }
        }

        public string ConvertToThisPixelFormat
        {
            get
            {
                return _convertToThisPixelFormat;
            }
            set
            {
                _convertToThisPixelFormat = value;
            }
        }
        string _convertToThisPixelFormat;

        public string FilterType //must always be set to "DLL" in case of DLL filter
        {
            get
            {
                return _filterType;
            }
            set
            {
                _filterType = value;
            }
        }
        string _filterType;

        public string PathToDll//same as FileName
        {
            get
            {
                return _pathToDll;
            }
            set
            {
                _pathToDll = value;
            }
        }
        string _pathToDll;

        public string FilterGuid
        {
            get
            {
                return _filterGuid;
            }
            set
            {
                _filterGuid = value;
            }
        }
        string _filterGuid;

        public string DefaultParameters
        {
            get 
            { 
                return _defaultParameters; 
            }
            set 
            { 
                _defaultParameters = value; 
            }
        }

        public string Version
        {
            get
            {
                return _version;
            }
        }
        string _version;
        #endregion
    }
}
