using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using CustomFilterExample1;

namespace GeneralCustomFilter
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        private void btnBrowseImage_Click(object sender, EventArgs e)
        {
            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.InitialDirectory = System.IO.Directory.GetCurrentDirectory();
            fileDialog.Title = "Select an image!";
            fileDialog.Filter = "All Files (*.*)|*.*";
            fileDialog.FilterIndex = 0;
            fileDialog.RestoreDirectory = false;

            if (fileDialog.ShowDialog() == DialogResult.OK)
            {
                txtBrowseImage.Text = fileDialog.FileName;
            }
        }

        private void btnLoadImage_Click(object sender, EventArgs e)
        {
            try
            {

                if (File.Exists(txtBrowseImage.Text))
                {
                    Bitmap image = (Bitmap)Image.FromFile(txtBrowseImage.Text);
                    if (image.PixelFormat == System.Drawing.Imaging.PixelFormat.Format1bppIndexed ||
                        image.PixelFormat == System.Drawing.Imaging.PixelFormat.Format4bppIndexed ||
                        image.PixelFormat == System.Drawing.Imaging.PixelFormat.Format8bppIndexed)
                        image = new Bitmap(image);

                    pictureBox1.Image = image;
                }
                else
                {
                    MessageBox.Show("There is no image on the selected path!" + Environment.NewLine);
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show("Could not load image from selected path!" + Environment.NewLine + ex.Message + Environment.NewLine+ ex.StackTrace);
            }
        }

        private void btnFilterIt_Click(object sender, EventArgs e)
        {
            try
            {
                Bitmap image = (Bitmap)pictureBox1.Image;
                if (image != null)
                {
                    try
                    {
                        //CustomFilterExample1.CustomFilterExample1.
                        CustomFilterExample1.CustomFilterExample1 filterExample1 = new CustomFilterExample1.CustomFilterExample1();
                        Bitmap filteredImage = filterExample1.RunFilter(image, "1");
                        if (filteredImage != null)
                        {
                            pictureBox1.Image = filteredImage;
                        }
                        else
                        {
                            if (!filterExample1.ErrorMessage.Equals(String.Empty))
                            {
                                MessageBox.Show(filterExample1.ErrorMessage);
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        //Just in case the Filter developer did not add a try catch on the Run Filter method
                        MessageBox.Show("The RunFilter method threw an exception! Image was not filtered" + Environment.NewLine + ex.Message + Environment.NewLine + ex.StackTrace);
                    }
                }
                else
                {
                    MessageBox.Show("There is no image to filter");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Could not filter image!" + Environment.NewLine + ex.Message + Environment.NewLine + ex.StackTrace);
            }
        }
    }
}